# DPM daily maintenance script
#
# Version 1.0
# 7/22/2009
#
# Jeff Graves
#

param ($replicaIncreaseRatio, $scIncreaseRatio, $replicaIncreaseSize, $scIncreaseSize)

if (!(Get-PSSnapin microsoft.dataprotectionmanager.powershell)) {
	write-host "Adding PS Snapin for DPM..." -Foreground Green
	add-PSSnapin Microsoft.DataProtectionManager.Powershell
}

$alertCol = @()
$DPMServer = hostname
$replicaFixedIncrease = $false
$scFixedIncrease = $false

if (!$replicaIncreaseRatio) {
	if(!$replicaIncreaseSize) {
		$replicaIncreaseRatio = 1.1
	}
	else {
		$replicaFixedIncrease
	}
}

if (!$scIncreaseRatio) {
	if(!$scIncreaseSize) {
		$scIncreaseRatio = 1.1
	}
	else {
		$scFixedIncrease
	}
}

function New-SQLconnection {
	Param ([string]$server, [string]$database = "master", [string]$connectionName = $ScriptName)
	if (test-path variable:\conn) {
		$conn.close()
	} else {
		$conn = new-object ('System.Data.SqlClient.SqlConnection')
	}
	$connString = "Server=$server;Integrated Security=SSPI;Database=$database;Application Name=$connectionName"
	$conn.ConnectionString = $connString
	$conn.StatisticsEnabled = $true
	$conn.Open()
	$conn
}

function Remove-SQLconnection {
	Param ($connection)
	$connection.close()
	$connection = $null
}

function Close-SQLquery {
	Param ($query)
	$query.close()
	$query = $null
}

function Query-SQL {
	Param ($query, $conn, [int]$CommandTimeout = 30)
	$sqlCmd = New-Object System.Data.SqlClient.SqlCommand
	$sqlCmd.CommandTimeout = $CommandTimeout
	$sqlCmd.CommandText = $query
	$sqlCmd.Connection = $conn
	$Rset = $sqlCmd.ExecuteReader()
	if ($? -eq $false) {
		Close-SQLquery $rset
		[int]$rset = -1
	}
	if ($Rset.hasrows -eq $false) {
		Close-SQLquery $rset
		[string]$rset = ""
	}
	,$Rset ## The comma is used to create an outer array, which PS strips off automatically when returning the $Rset
}

function NonQuery-SQL {
	Param ($query, $conn, $CommandTimeout = 30)
	$sqlCmd = New-Object System.Data.SqlClient.SqlCommand
	$sqlCmd.CommandTimeout = $CommandTimeout
	$sqlCmd.CommandText = $query
	$sqlCmd.Connection = $conn
	$RowsAffected = $sqlCmd.ExecuteNonQuery()
	if ($? -eq $false) {
		$RowsAffected = -2
	}
	$RowsAffected
}

function ParseXML {
	Param($XMLData)
	[System.Xml.XmlDocument] $xmlDoc = new-object System.Xml.XmlDocument
	$XMLDoc.LoadXML($XMLData)
	
}

#Generate SQL Connection
$ScriptName = $myInvocation.MyCommand.Name
$Server = "$DPMServer\MS`$DPM2007`$"
$myConn = New-SQLconnection $Server "DPMDB"
if ($myConn.state -eq "closed") {
	write-host "Failed to establish a connection to $DPMServer\MS`$DPM2007`$"
}
else {
	$query = "select al.alertid, al.type, al.severity, al.errorXML, al.resolution from tbl_AHP_Alerts al where al.ResolvedTime is null and al.type in (25,27,31,36)"
	$data = Query-SQL $query $myConn
	if ($data.gettype() -eq [int]) {
		write-host "Failed to query SQL server $DPMServer\MS`$DPM2007`$"
	}
	elseif ($data.gettype() -ne [string]) {
		while($data.Read()) {
			#Parse SQL output
			$dpmAlert = New-Object System.Object
			$dpmAlert | Add-Member -type NoteProperty -name AlertID -value ([string] $data.GetValue(0))
			$dpmAlert | Add-Member -type NoteProperty -name AlertType -value ([int] $data.GetValue(1))
			$dpmAlert | Add-Member -type NoteProperty -name Severity -value ([int] $data.GetValue(2))
			$errorXML = [string] $data.GetValue(3)

			[System.Xml.XmlDocument] $xmlDoc = new-object System.Xml.XmlDocument
			$XMLDoc.LoadXML($errorXML)
			
			#$dpmAlert | Add-Member -type NoteProperty -name ErrorXML -value $errorXML
			$dpmAlert | Add-Member -type NoteProperty -name DataSourceID -value ($XMLDoc.ErrorInfo.Parameter | where {$_.name -match "datasourceid"}).Value
			$dpmAlert | Add-Member -type NoteProperty -name ServerName -value ($XMLDoc.ErrorInfo.Parameter | where {$_.name -match "servername"}).Value
			$dpmAlert | Add-Member -type NoteProperty -name DataSource -value ($XMLDoc.ErrorInfo.Parameter | where {$_.name -match "datasourcename"}).Value
			$dpmAlert | Add-Member -type NoteProperty -name ProtectionGroup -value ($XMLDoc.ErrorInfo.Parameter | where {$_.name -match "protectedgroup"}).Value
			$dpmAlert | Add-Member -type NoteProperty -name Resolution -value ([int] $data.GetValue(4))

			$alertCol += $dpmAlert
		}
		Close-SQLquery $data
	}
}

$alertCol = $alertCol | sort @{expression="Servername";Ascending=$true},@{expression="DataSource";Ascending=$true},@{expression="Severity";Descending=$true},@{expression="AlertType";Descending=$true}

foreach($alert in $alertCol) {
	$AlertID = $alert.AlertID
	$ProtectionGroupName = $alert.ProtectionGroup
	$DatasourceName = $alert.DataSource
	$DataSourceID = $alert.DataSourceID
	$ServerName = $alert.ServerName
	switch ($alert.AlertType) {
		25 {
			#recovery point creation failed
			write-host "Getting $DatasourceName on $ServerName from $ProtectionGroupName..." 
			$ds = Get-Datasource $DPMServer | where { $_.Id -eq $DataSourceID } 
			if (!$ds) {
    				write-host "Failed To get datasource::$DatasourceName" -foreground Red
			}
			else {
				write-host "Creating Recovery Point..."
				$job = New-RecoveryPoint -Datasource $ds -Disk -DiskRecoveryPointOption withsynchronize

				if ($job) {
					#Inactivate Alert
					$date = Get-Date -format 'yyyy-MM-dd hh:mm:ss'
					$query = "exec dbo.prc_AHP_Alert_Resolve @AlertID = '" + $AlertID + "' , @ResolvedTime = '" + $date + "', @Resolution = 2"
					$data = NonQuery-SQL $query $myConn
					if ($data -eq -2) {
						Write-Host "Failed to inactivate alert::$AlertID" -foreground Red
					}
				}
				else {
					write-host "Failed to create recovery point::$DatasourceName" -foreground Red
				}
			}
		}
		27 {
			#replica is inconsistent
			write-host "Getting $DatasourceName on $ServerName from $ProtectionGroupName..." 
			$ds = Get-Datasource $DPMServer | where { $_.Id -eq $DataSourceID } 
			if (!$ds) {
    				write-host "Failed To get datasource::$DatasourceName" -foreground Red
			}
			else {
				write-host "Initiating Consistency Check..."
				$job = Start-DatasourceConsistencyCheck -Datasource $ds 

				if($job) {
					#Inactivate Alert
					$date = Get-Date -format 'yyyy-MM-dd hh:mm:ss'
					$query = "exec dbo.prc_AHP_Alert_Resolve @AlertID = '" + $AlertID + "' , @ResolvedTime = '" + $date + "', @Resolution = 2"
					$data = NonQuery-SQL $query $myConn
					if ($data -eq -2) {
						Write-Host "Failed to inactivate alert::$AlertID" -foreground Red
					}
				}
				else {
					write-host "Failed to start replica consistency check::$DatasourceName" -foreground Red
				}
			}
		}
		31 {
			#replica disk threshold exceeded
			$ProtectionGroup = Get-ProtectionGroup $DPMServer | where {$_.FriendlyName -eq $ProtectionGroupName}

			if (!$ProtectionGroup) {
    				write-host "Failed To get protection group::$ProtectionGroupName" -foreground Red
			}
			else {
				$MPG = Get-ModifiableProtectionGroup $ProtectionGroup
				write-host "Getting $DatasourceName on $ServerName from $ProtectionGroupName..." 
				$ds = Get-Datasource $MPG | where { $_.Id -eq $DataSourceID } 
				if (!$ds) {
    					write-host "Failed To get datasource::$DatasourceName" -foreground Red
				}
				else {
					if ($ds.ReplicaUsedSpace -eq $ds.ReplicaSize) {
						#The replica is full. Most likely a huge chunk of data was added and 
						#we should do a major resize based on the current dataset size.
						$ps = Get-ProductionServer $DPMServer | where {$ServerName -match $_.MachineName}
						#$ps_ds = Get-Datasource -ProductionServer $ps -Inquire | where {$_.Name -eq $DatasourceName}
						$ps_ds = Get-Datasource -ProductionServer $ps -Inquire | where { $_.Id -eq $DataSourceID }
						$da = Get-DatasourceDiskAllocation -Datasource $ps_ds
						$ReplicaSize = $da.OptimizedSize
					}
					else {
						$ReplicaSize = $ds.ReplicaSize
					}

					if(!$replicaFixedIncrease) {
						$NewReplicaSize = $ReplicaSize * $replicaIncreaseRatio
					}
					else {
						$NewReplicaSize = $ReplicaSize + $replicaIncreaseSize
					}

					$OldSize = "{0:N2}" -f ($ReplicaSize/1GB)
					$NewSize = "{0:N2}" -f ($NewReplicaSize/1GB)

					write-host "Modifying Replica Disk Allocation from $OldSize to $NewSize..."
					Set-DatasourceDiskAllocation -Manual -Datasource $ds -ProtectionGroup $MPG -ReplicaArea $NewReplicaSize

					Set-ProtectionGroup $MPG

					#Inactivate Alert
					$date = Get-Date -format 'yyyy-MM-dd hh:mm:ss'
					$query = "exec dbo.prc_AHP_Alert_Resolve @AlertID = '" + $AlertID + "' , @ResolvedTime = '" + $date + "', @Resolution = 2"
					$data = NonQuery-SQL $query $myConn
					if ($data -eq -2) {
						Write-Host "Failed to inactivate alert::$AlertID" -foreground Red
					}
				}
			}
		}
		36 {
			#recovery point volume threshold exceeded
			$ProtectionGroup = Get-ProtectionGroup $DPMServer | where {$_.FriendlyName -eq $ProtectionGroupName}

			if (!$ProtectionGroup) {
    				write-host "Failed To get protection group::$ProtectionGroupName" -foreground Red
			}
			else {
				$MPG = Get-ModifiableProtectionGroup $ProtectionGroup
				write-host "Getting $DatasourceName on $ServerName from $ProtectionGroupName..." 
				$ds = Get-Datasource $MPG | where { $_.Id -eq $DataSourceID } 
				if (!$ds) {
    					write-host "Failed To get datasource::$DatasourceName" -foreground Red
				}
				else {
					$SCSize = $ds.ShadowCopyAreaSize

					if(!$scFixedIncrease) {
						$NewSCSize = $SCSize * $scIncreaseRatio
					}
					else {
						$NewSCSize = $SCSize + $scIncreaseSize
					}

					$OldSize = "{0:N2}" -f ($SCSize/1GB)
					$NewSize = "{0:N2}" -f ($NewSCSize/1GB)


					write-host "Modifying Recovery Point Disk Allocation from $OldSize to $NewSize..."
					Set-DatasourceDiskAllocation -Manual -Datasource $ds -ProtectionGroup $MPG -ShadowCopyArea $NewSCSize

					Set-ProtectionGroup $MPG

					#Inactivate Alert
					$date = Get-Date -format 'yyyy-MM-dd hh:mm:ss'
					$query = "exec dbo.prc_AHP_Alert_Resolve @AlertID = '" + $AlertID + "' , @ResolvedTime = '" + $date + "', @Resolution = 2"
					$data = NonQuery-SQL $query $myConn
					if ($data -eq -2) {
						Write-Host "Failed to inactivate alert::$AlertID" -foreground Red
					}
				}
			}
		}
	}		
}

Remove-SQLconnection $myConn